searchData={"content_type":"text/markdown","items":[{"doc":"Extend `ExUnit.Case` test cases with feature tests.\n\nAt the top of your test module:\n\n```\ndefmodule MyApp.FeatureTests do\n  use ExUnit.Case, async: true\n  use Given.Case\nend\n\n```\nA feature is often defined as a set of scenarios. The feature can be achieved\nby grouping scenarios inside a standard ExUnit `describe` block:\n\n```elixir\ndescribe \"feature 1\" do\n  scenario \"one\", ~s[Given x When y Then z]\nend\n\ndescribe \"feature 2\" do\n  scenario \"two\", ~s\"\"\n  Given x\n  When y\n  Then z\n  \"\"\nend\n```\n\n^ note the doc does not use triple quotes - your test cases should!\n\nA scenario is a list of clauses. Each clause must start with one of\nGIVEN, WHEN, or THEN in UPPER or Title case. Newlines are counted as\nwhitespace when separating clauses and are recommended but not required.\n\nClauses must appear in the order GIVEN WHEN THEN. If multiple clauses of the\nsame kind are required then AND (or And) can be interspersed:\n\n```\nGIVEN pre-condition AND another pre-condition\nWHEN action\nTHEN post-condition AND another post-condition\n```\n\nThe error messages on the parser are not yet very descriptive! If there is\na problem then a `Given.SyntaxError` will be raised when the test is run\nhowever you may also fail the compile with this flag:\n\n    use Given.Case, fail_compile: true\n\nIf a scenario is required but has not yet been written you can write a\nfailing test as you would in `ExUnit.Case` by writing a placeholder with a\nname but without the prose:\n\n```elixir\ndescribe \"Leap seconds\" do\n  scenario \"handle 61 seconds in a minute\"\nend\n```\n\nOnce the scenario is written you must implement the `Given.Step` callbacks\nthat match the parsed terms:\n\n```elixir\ndescribe \"elixir-lang.org\" do\n  scenario \"peek\", ~s\"\"\n  Given the string \"Elixir\"\n  When graphemes\n  And frequencies\n  Then \"i\" occurs 2 times\n  And \"E\" occurs 1 time\n  \"\"\nend\n\n# Pre-conditions\ndef given_({:the_string, s}, _), do: [str: s]\n# Actions\ndef when_({:graphemes}, %{str: s}), do: [gs: String.graphemes(s)]\ndef when_({:frequencies}, %{gs: gs}), do: [freq: Enum.frequencies(gs)]\n# Post-conditions\ndef then_({l, :occurs, n, _}, %{freq: freq}), do: assert n == freq[l]\n```\n\nA pre-condition may return a keyword list of values that are appended to\nthe context and passed from clause to clause. `ExUnit.Assertions` can be\nused as normal in the post-conditions.\n\nYou may setup a test context as normal:\n\n```elixir\nsetup [:setup_a]\n\nscenario \"addition\", ~s\"\"\nGiven :a\nThen :a is 1\n\"\"\n\ndef given_(_, _), do: true\ndef then_({_, :is, x}, %{a: a}), do: assert a == x\n\ndef setup_a(_context) do\n  [a: 1]\nend\n```","ref":"Given.Case.html","title":"Given.Case","type":"module"},{"doc":"Defines a named yet not implemented test.\n\nProvides a convenient macro that allows a test to be defined\nwith a string, but not yet implemented. The resulting test will\nalways fail and print a \"Not implemented\" error message. The\nresulting test case is also tagged with `:not_implemented`.","ref":"Given.Case.html#scenario/1","title":"Given.Case.scenario/1","type":"macro"},{"doc":"scenario \"handle leap seconds\"","ref":"Given.Case.html#scenario/1-examples","title":"Examples - Given.Case.scenario/1","type":"macro"},{"doc":"Defines an ExUnit test with given name and prose.\n\nThe test context cannot be matched but is passed into the first clause. For\nmore information on contexts, see `ExUnit.Callbacks`.\n\nClauses must appear in the order GIVEN WHEN THEN. If multiple clauses of the\nsame kind are required then AND (or And) can be interspersed:","ref":"Given.Case.html#scenario/2","title":"Given.Case.scenario/2","type":"macro"},{"doc":"GIVEN pre-condition AND another pre-condition\n    WHEN action\n    THEN post-condition AND another post-condition","ref":"Given.Case.html#scenario/2-example","title":"Example - Given.Case.scenario/2","type":"macro"},{"doc":"Parse a textual scenario into a keyword list of clauses.\n\nSee `Given.Case` for a description on how to structure your features and\nscenarios, what follows are the rules inside a scenario.\n\nA scenario is a list of clauses. Each clause must start with one of\nGIVEN, WHEN, or THEN in UPPER or Title case. The remainder of a clause is\nsplit into terms and passed to your code as a tuple.\n\nThere are some restrictions on the terms supported - more flexibility and\nmore terms will be added.\n\nThe following terms are supported:\n\n* atoms - only simple roman alphabet with colon prefix e.g. `:abc`\n* dates - ISO-8601 in the format `YYYY-MM-DD` becomes `Date`\n* floats\n* hexadecimal - with `0x` prefix e.g. `0xFE` is 254\n* integer - positive and negative base 10 without separators\n* range - a dashed pair of positive integers e.g. `1-6` becomes `1..6`\n* string - any unicode chars within double quotes (no escaping of quotes)\n* time - ISO-8601 in the format `HH:MM:SS` becomes `Time`\n* words - only using the Roman alphabet (temporary restriction!)\n\nNot currently supported but planned:\n* atoms with extended alphabet\n* date times\n* lists\n* time locale\n* words using any alphabet\n\nAny consecutive words will be collapsed into a single atom. For example\nthe words \"the cat sat on the mat\" becomes `:the_cat_sat_on_the_mat`\n\nThe clause:\n\n    \"Elîxir 1.0\" was released 2014-09-18\n\nbecomes the tuple:\n\n    {\"Elîxir 1.0\", :was_released, ~D[2014-09-18]}","ref":"Given.Parser.html","title":"Given.Parser","type":"module"},{"doc":"Parse a textual scenario into a keyword list of clauses","ref":"Given.Parser.html#parse/1","title":"Given.Parser.parse/1","type":"function"},{"doc":"Parse a textual scenario into a keyword list of clauses.\n\nIntended for internal use - expects the `__CALLER__` struct to be supplied\nand returns error constructor in case of syntax error.","ref":"Given.Parser.html#parse!/2","title":"Given.Parser.parse!/2","type":"function"},{"doc":"","ref":"Given.Parser.html#t:input/0","title":"Given.Parser.input/0","type":"type"},{"doc":"Behaviour that documents the return values of a Given, When or Then step.","ref":"Given.Step.html","title":"Given.Step","type":"behaviour"},{"doc":"Setup the state of the world.\n\nNormally returns a keyword list of one or more values to be\nappended to the context for future steps.\n\nNote the trailing underscore to be consistent with `when_`.","ref":"Given.Step.html#c:given_/2","title":"Given.Step.given_/2","type":"callback"},{"doc":"Run assertions on the context.\n\nNormally would not change the context so return the result of an assertion.\nNote the trailing underscore to be consistent with `when_`.","ref":"Given.Step.html#c:then_/2","title":"Given.Step.then_/2","type":"callback"},{"doc":"Execute the behaviour.\n\nNormally returns a keyword list of one or more values to be\nappended to the context for future steps.\n\n`when` is a reserved keyword in Elixir and so we use `when_`.","ref":"Given.Step.html#c:when_/2","title":"Given.Step.when_/2","type":"callback"},{"doc":"ExUnit test case context","ref":"Given.Step.html#t:context/0","title":"Given.Step.context/0","type":"type"},{"doc":"Return of each step has the following effect on the context:\n* :error - fail with a RuntimeError\n* {:error, _} - fail with a RuntimeError\n* false - fail with a RuntimeError\n* fn/1 - applies function to the context returning new context\n* keyword list - put the items into the context\n* other - ignored and context is not changed\n* true - context is not changed","ref":"Given.Step.html#t:result/0","title":"Given.Step.result/0","type":"type"},{"doc":"Tuple representing the parsed clause","ref":"Given.Step.html#t:step/0","title":"Given.Step.step/0","type":"type"},{"doc":"","ref":"Given.SyntaxError.html","title":"Given.SyntaxError","type":"exception"}]}